@@ -1,18 +1,31 @@
 //! Points in the affine space.
 //!
 //! Points represent absolute positions in the space.
-//! So in conmtrast to vectors it doesn't make sense to add two points,
+//! So in contrast to vectors it doesn't make sense to add two points,
 //! or to multiply a point by a scalar value.
 //!
 //! A vector can be added to a point, representing a move in the space.
 //! Or conversely, two points can be subtracted to get back the vector
 //! representing the move between them.
 //!
+//! Division and remainder are implemented for points divided by a bounding box.
+//! The remainder is the mapping of any point into the bounding box,
+//! assuming that the space is filled with a regular grid of bounding boxes
+//! of the same size.  This allows to easily implement toroidal manifolds,
+//! where objects leaving the bounding box at the maximum for a coordinate
+//! reenter at the minimum or vice versa.
+//! This can be implemented by taking the remainder after every operation,
+//! similarly to computation modulo on numbers.
+//! The associated division operation tells the instance of the bounding box
+//! the point is in.
+//!
 //! This module implements 2d, 3d and 4d points.
 
 use core::cmp::Ordering;
 use core::fmt;
 use core::marker::PhantomData;
+use core::ops::Div;
+use core::ops::Rem;
 
 use core::ops::Add;
 use core::ops::AddAssign;
@@ -23,6 +36,7 @@ use core::ops::SubAssign;
 use crate::v2d;
 use crate::v3d;
 use crate::v4d;
+use crate::BBox;
 use crate::Integer;
 use crate::Vec2d;
 use crate::Vec3d;
@@ -142,14 +156,30 @@ where
         (self - other).norm_l_infty()
     }
 
+    /// Returns the square of the distance to another point w.r.t. the L2 norm.
+    ///
+    /// This is the standard Euclidean distance used in continuous settings.
+    /// We use the square to avoid the square root and get discrete values.
+    ///
+    /// # Example
+    /// ```
+    /// # use lowdim::p4d;
+    /// let p0 = p4d(1, -2, 7, -4);
+    /// let p1 = p4d(2, -3, 7, 4);
+    /// assert_eq!(66, p0.distance_l2_squared(p1));
+    /// ```
+    pub fn distance_l2_squared(&self, other: Self) -> S {
+        (self - other).norm_l2_squared()
+    }
+
     /// Creates a vector containing the orthogonal neighbors of a point.
     pub fn neighbors_l1(&self) -> impl Iterator<Item = Self> {
-        NeighBorsIter::new(*self, V::unit_vecs_l1())
+        NeighborsIter::new(*self, V::unit_vecs_l1())
     }
 
     /// Creates a vector containing the orthogonal and diagonal neighbors of a point.
     pub fn neighbors_l_infty(&self) -> impl Iterator<Item = Self> {
-        NeighBorsIter::new(*self, V::unit_vecs_l_infty())
+        NeighborsIter::new(*self, V::unit_vecs_l_infty())
     }
 
     /// Returns the componentwise partial ordering for this and another point.
@@ -481,6 +511,96 @@ where
     }
 }
 
+impl<S, V> Div<BBox<S, V>> for Point<S, V>
+where
+    S: Integer,
+    V: Vector<S>,
+{
+    type Output = V;
+
+    fn div(self, bbox: BBox<S, V>) -> V {
+        (self - bbox.min()) / bbox.lengths()
+    }
+}
+impl<'a, S, V> Div<&'a BBox<S, V>> for Point<S, V>
+where
+    S: Integer,
+    V: Vector<S>,
+{
+    type Output = V;
+
+    fn div(self, bbox: &'a BBox<S, V>) -> V {
+        (self - bbox.min()) / bbox.lengths()
+    }
+}
+impl<'a, S, V> Div<BBox<S, V>> for &'a Point<S, V>
+where
+    S: Integer,
+    V: Vector<S>,
+{
+    type Output = V;
+
+    fn div(self, bbox: BBox<S, V>) -> V {
+        (self - bbox.min()) / bbox.lengths()
+    }
+}
+impl<'a, S, V> Div<&'a BBox<S, V>> for &'a Point<S, V>
+where
+    S: Integer,
+    V: Vector<S>,
+{
+    type Output = V;
+
+    fn div(self, bbox: &'a BBox<S, V>) -> V {
+        (self - bbox.min()) / bbox.lengths()
+    }
+}
+
+impl<S, V> Rem<BBox<S, V>> for Point<S, V>
+where
+    S: Integer,
+    V: Vector<S>,
+{
+    type Output = Point<S, V>;
+
+    fn rem(self, bbox: BBox<S, V>) -> Point<S, V> {
+        bbox.min() + (self - bbox.min()) % bbox.lengths()
+    }
+}
+impl<'a, S, V> Rem<&'a BBox<S, V>> for Point<S, V>
+where
+    S: Integer,
+    V: Vector<S>,
+{
+    type Output = Point<S, V>;
+
+    fn rem(self, bbox: &'a BBox<S, V>) -> Point<S, V> {
+        bbox.min() + (self - bbox.min()) % bbox.lengths()
+    }
+}
+impl<'a, S, V> Rem<BBox<S, V>> for &'a Point<S, V>
+where
+    S: Integer,
+    V: Vector<S>,
+{
+    type Output = Point<S, V>;
+
+    fn rem(self, bbox: BBox<S, V>) -> Point<S, V> {
+        bbox.min() + (self - bbox.min()) % bbox.lengths()
+    }
+}
+impl<'a, S, V> Rem<&'a BBox<S, V>> for &'a Point<S, V>
+where
+    S: Integer,
+    V: Vector<S>,
+{
+    type Output = Point<S, V>;
+
+    fn rem(self, bbox: &'a BBox<S, V>) -> Point<S, V> {
+        bbox.min() + (self - bbox.min()) % bbox.lengths()
+    }
+}
+
 impl<S, V> AddAssign<V> for Point<S, V>
 where
     S: Integer,
@@ -520,7 +640,7 @@ where
 }
 
 #[derive(Clone, Debug)]
-struct NeighBorsIter<S, V, I>
+struct NeighborsIter<S, V, I>
 where
     S: Integer,
     V: Vector<S>,
@@ -529,17 +649,17 @@ where
     p: Point<S, V>,
     iter: I,
 }
-impl<S, V, I> NeighBorsIter<S, V, I>
+impl<S, V, I> NeighborsIter<S, V, I>
 where
     S: Integer,
     V: Vector<S>,
     I: ExactSizeIterator<Item = V>,
 {
     fn new(p: Point<S, V>, iter: I) -> Self {
-        NeighBorsIter { p, iter }
+        NeighborsIter { p, iter }
     }
 }
-impl<S, V, I> Iterator for NeighBorsIter<S, V, I>
+impl<S, V, I> Iterator for NeighborsIter<S, V, I>
 where
     S: Integer,
     V: Vector<S>,
@@ -553,7 +673,7 @@ where
         self.iter.size_hint()
     }
 }
-impl<S, V, I> ExactSizeIterator for NeighBorsIter<S, V, I>
+impl<S, V, I> ExactSizeIterator for NeighborsIter<S, V, I>
 where
     S: Integer,
     V: Vector<S>,
@@ -566,6 +686,7 @@ mod tests {
     use core::cmp::Ordering;
     use core::convert::TryFrom;
 
+    use crate::bb2d;
     use crate::p2d;
     use crate::p3d;
     use crate::p4d;
@@ -742,6 +863,26 @@ mod tests {
         assert_eq!(v2d(-1, -6), &p - &q);
     }
 
+    #[test]
+    fn test_div_pb() {
+        let p = p2d(2, 1);
+        let b = bb2d(3..=5, 4..=9);
+        assert_eq!(v2d(-1, -1), p / b);
+        assert_eq!(v2d(-1, -1), p / &b);
+        assert_eq!(v2d(-1, -1), &p / b);
+        assert_eq!(v2d(-1, -1), &p / &b);
+    }
+
+    #[test]
+    fn test_rem_pb() {
+        let p = p2d(2, 1);
+        let b = bb2d(3..=5, 4..=9);
+        assert_eq!(p2d(5, 7), p % b);
+        assert_eq!(p2d(5, 7), p % &b);
+        assert_eq!(p2d(5, 7), &p % b);
+        assert_eq!(p2d(5, 7), &p % &b);
+    }
+
     #[test]
     fn test_add_assign() {
         let mut u = p2d(2, 1);
@@ -802,6 +943,63 @@ mod tests {
         assert_eq!(p4d(3, 7, 5, 4), u.max(v));
     }
 
+    #[test]
+    fn test_distance_l1_2d() {
+        let u = p2d(2, 7);
+        let v = p2d(3, 1);
+        assert_eq!(7, u.distance_l1(v));
+    }
+    #[test]
+    fn test_distance_l1_3d() {
+        let u = p3d(2, 7, 5);
+        let v = p3d(3, 1, 1);
+        assert_eq!(11, u.distance_l1(v));
+    }
+    #[test]
+    fn test_distance_l1_4d() {
+        let u = p4d(2, 7, 5, -4);
+        let v = p4d(3, 1, 1, 4);
+        assert_eq!(19, u.distance_l1(v));
+    }
+
+    #[test]
+    fn test_distance_l_infty_2d() {
+        let u = p2d(2, 7);
+        let v = p2d(3, 1);
+        assert_eq!(6, u.distance_l_infty(v));
+    }
+    #[test]
+    fn test_distance_l_infty_3d() {
+        let u = p3d(2, 7, 5);
+        let v = p3d(3, 1, 1);
+        assert_eq!(6, u.distance_l_infty(v));
+    }
+    #[test]
+    fn test_distance_l_infty_4d() {
+        let u = p4d(2, 7, 5, -4);
+        let v = p4d(3, 1, 1, 4);
+        assert_eq!(8, u.distance_l_infty(v));
+    }
+
+    #[test]
+    fn test_distance_l2_squared_2d() {
+        let u = p2d(2, 7);
+        let v = p2d(3, 1);
+        assert_eq!(37, u.distance_l2_squared(v));
+    }
+    #[test]
+    fn test_distance_l2_squared_3d() {
+        let u = p3d(2, 7, 5);
+        let v = p3d(3, 1, 1);
+        assert_eq!(53, u.distance_l2_squared(v));
+    }
+    #[test]
+    fn test_distance_l2_squared_4d() {
+        let u = p4d(2, 7, 5, -4);
+        let v = p4d(3, 1, 1, 4);
+        assert_eq!(117, u.distance_l2_squared(v));
+    }
+
     #[test]
     fn test_neighbors_l1_2d() {
         let p = p2d(2, -3);
