@@ -9,6 +9,7 @@ use core::ops::Div;
 use core::ops::Index;
 use core::ops::Mul;
 use core::ops::Neg;
+use core::ops::Rem;
 use core::ops::Sub;
 use core::ops::SubAssign;
 
@@ -384,28 +385,115 @@ impl<S: Integer> Div<S> for Vec2d<S> {
     type Output = Vec2d<S>;
 
     fn div(self, other: S) -> Vec2d<S> {
-        Vec2d::with(|i| self[i] / other)
+        Vec2d::with(|i| self[i].div_euclid(other))
     }
 }
 impl<'a, S: Integer> Div<&'a S> for Vec2d<S> {
     type Output = Vec2d<S>;
 
     fn div(self, other: &'a S) -> Vec2d<S> {
-        Vec2d::with(|i| self[i] / other)
+        Vec2d::with(|i| self[i].div_euclid(*other))
     }
 }
 impl<'a, S: Integer> Div<S> for &'a Vec2d<S> {
     type Output = Vec2d<S>;
 
     fn div(self, other: S) -> Vec2d<S> {
-        Vec2d::with(|i| self[i] / other)
+        Vec2d::with(|i| self[i].div_euclid(other))
     }
 }
 impl<'a, S: Integer> Div<&'a S> for &'a Vec2d<S> {
     type Output = Vec2d<S>;
 
     fn div(self, other: &'a S) -> Vec2d<S> {
-        Vec2d::with(|i| self[i] / other)
+        Vec2d::with(|i| self[i].div_euclid(*other))
+    }
+}
+
+impl<S: Integer> Rem<S> for Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn rem(self, other: S) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].rem_euclid(other))
+    }
+}
+impl<'a, S: Integer> Rem<&'a S> for Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn rem(self, other: &'a S) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].rem_euclid(*other))
+    }
+}
+impl<'a, S: Integer> Rem<S> for &'a Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn rem(self, other: S) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].rem_euclid(other))
+    }
+}
+impl<'a, S: Integer> Rem<&'a S> for &'a Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn rem(self, other: &'a S) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].rem_euclid(*other))
+    }
+}
+
+impl<S: Integer> Div<Vec2d<S>> for Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn div(self, other: Vec2d<S>) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].div_euclid(other[i]))
+    }
+}
+impl<'a, S: Integer> Div<&'a Vec2d<S>> for Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn div(self, other: &'a Vec2d<S>) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].div_euclid(other[i]))
+    }
+}
+impl<'a, S: Integer> Div<Vec2d<S>> for &'a Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn div(self, other: Vec2d<S>) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].div_euclid(other[i]))
+    }
+}
+impl<'a, S: Integer> Div<&'a Vec2d<S>> for &'a Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn div(self, other: &'a Vec2d<S>) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].div_euclid(other[i]))
+    }
+}
+
+impl<S: Integer> Rem<Vec2d<S>> for Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn rem(self, other: Vec2d<S>) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].rem_euclid(other[i]))
+    }
+}
+impl<'a, S: Integer> Rem<&'a Vec2d<S>> for Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn rem(self, other: &'a Vec2d<S>) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].rem_euclid(other[i]))
+    }
+}
+impl<'a, S: Integer> Rem<Vec2d<S>> for &'a Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn rem(self, other: Vec2d<S>) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].rem_euclid(other[i]))
+    }
+}
+impl<'a, S: Integer> Rem<&'a Vec2d<S>> for &'a Vec2d<S> {
+    type Output = Vec2d<S>;
+
+    fn rem(self, other: &'a Vec2d<S>) -> Vec2d<S> {
+        Vec2d::with(|i| self[i].rem_euclid(other[i]))
     }
 }
 
@@ -649,11 +737,40 @@ mod tests {
 
     #[test]
     fn test_div_vs() {
-        let v = v2d(6, 14);
-        assert_eq!(v2d(3, 7), v / 2);
-        assert_eq!(v2d(3, 7), &v / 2);
-        assert_eq!(v2d(3, 7), v / &2);
-        assert_eq!(v2d(3, 7), &v / &2);
+        let v = v2d(-5, 14);
+        assert_eq!(v2d(-2, 4), v / 3);
+        assert_eq!(v2d(-2, 4), &v / 3);
+        assert_eq!(v2d(-2, 4), v / &3);
+        assert_eq!(v2d(-2, 4), &v / &3);
+    }
+
+    #[test]
+    fn test_rem_vs() {
+        let v = v2d(-5, 14);
+        assert_eq!(v2d(1, 2), v % 3);
+        assert_eq!(v2d(1, 2), &v % 3);
+        assert_eq!(v2d(1, 2), v % &3);
+        assert_eq!(v2d(1, 2), &v % &3);
+    }
+
+    #[test]
+    fn test_div_vv() {
+        let v0 = v2d(-5, 14);
+        let v1 = v2d(3, 5);
+        assert_eq!(v2d(-2, 2), v0 / v1);
+        assert_eq!(v2d(-2, 2), &v0 / v1);
+        assert_eq!(v2d(-2, 2), v0 / &v1);
+        assert_eq!(v2d(-2, 2), &v0 / &v1);
+    }
+
+    #[test]
+    fn test_rem_vv() {
+        let v0 = v2d(-5, 14);
+        let v1 = v2d(3, 5);
+        assert_eq!(v2d(1, 4), v0 % v1);
+        assert_eq!(v2d(1, 4), &v0 % v1);
+        assert_eq!(v2d(1, 4), v0 % &v1);
+        assert_eq!(v2d(1, 4), &v0 % &v1);
     }
 
     #[test]
